"""
Type interface for the mailer module.
"""
from dataclasses import astuple, dataclass


@dataclass()
class Sender:
    uname: str
    passwd: str

    def __iter__(self):
        return iter(astuple(self))

    def __getitem__(self, keys):
        return iter(getattr(self, k) for k in keys)


@dataclass()
class Recipient:
    uname: str

    def __iter__(self):
        return iter(astuple(self))

    def __getitem__(self, keys):
        return iter(getattr(self, k) for k in keys)


@dataclass()
class MailArgs:
    sender: Sender
    recipient: Recipient
    subj: str
    content: str

    def __iter__(self):
        return iter(astuple(self))

    def __getitem__(self, keys):
        return iter(getattr(self, k) for k in keys)



"""Style declarations for log messages & other things."""

import fontawesome as fai
from chalky import Chalk, bg, fg, hex
from chalky.color import Color
from chalky.style import Style

# Too much custom
ghost = hex("#f8f8ff", background=False)
gray65 = hex("#a6a6a6", background=False)
gray09 = hex("#171717")

# Composition
fg = {
    "w": gray65,  # Chalk(foreground=Color.WHITE),
    "g": Chalk(foreground=Color.BRIGHT_GREEN),
    "r": Chalk(foreground=Color.RED),
    "b": Chalk(foreground=Color.BLUE),
}

bg = {"g9": gray09}

weight = {
    "it": Chalk(style={Style.ITALIC}),
    "b": Chalk(style={Style.BOLD}),
    "s": Chalk(style={Style.STRIKETHROUGH}),
    "t": Chalk(style={Style.SLOW_BLINK}),
}

# print(custom_rgb | "Potential link text")
# print(custom_hex | "Black on green text")

# Exports
get_andlog_fmt = lambda: fg["b"] & weight["b"]
get_orlog_fmt = lambda: fg["w"] + weight["it"] + bg["g9"]



"""
Nicely typed env mgmt.
"""
from dotenv import dotenv_values

from dataclasses import dataclass
from pydantic import BaseSettings

config = dotenv_values(".env")

__all__ = ["environment"]


class Common(BaseSettings):
    test: str = config["TEST"]


class Servers(BaseSettings):
    pass


class Databases(BaseSettings):
    pass


class Environment(Common, Servers, Databases):
    pass


environment = Environment()



"""
A comfier logging module.
"""

import logging
import sys
from logging import Logger, StreamHandler

from fontawesome import icons as fai

from louzlib.sty import get_orlog_fmt

logging.addLevelName(logging.DEBUG, f"d")
logging.addLevelName(logging.INFO, f"i")
logging.addLevelName(logging.WARNING, f"w")
logging.addLevelName(logging.ERROR, f"e")
logging.addLevelName(logging.NOTSET, f"n")
logging.addLevelName(logging.CRITICAL, f"c")


def get_logger(level: int = logging.DEBUG, app: str = "default") -> Logger:
    logr = logging.getLogger(app)
    logr.setLevel(level)
    return logr


def get_handler(level: int = logging.DEBUG) -> StreamHandler:
    handlr = StreamHandler(sys.stdout)
    handlr.setLevel(level)
    return handlr


def init_logger(level: int = logging.DEBUG, app: str = "default"):
    log4, handlr = get_logger(level, app), get_handler(level)

    fmt = logging.Formatter(
        get_orlog_fmt() | "[%(asctime)s - %(name)s - %(levelname)s] %(message)s",
        # Imagine if we could render a clock from  3 svgs just in time lol
        "%Y-%m-%d %H:%M:%S",
    )

    handlr.setFormatter(fmt)
    log4.addHandler(handlr)
    return log4



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['louzlib', 'louzlib.env', 'louzlib.log', 'louzlib.owl', 'louzlib.sty']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'fontawesome>=5.10.1.post1,<6.0.0',
 'lxml>=4.9.1,<5.0.0',
 'pandas>=1.4.4,<2.0.0',
 'ptpython>=3.0.20,<4.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'randomname>=0.1.5,<0.2.0',
 'requests>=2.28.1,<3.0.0',
 'typer>=0.6.1,<0.7.0']

setup_kwargs = {
    'name': 'louzlib',
    'version': '1.0.3',
    'description': 'like xanax but for the python.',
    'long_description': '# xanax\n',
    'author': 'lmisto',
    'author_email': 'louaimisto@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'http://github.com/lmist/louzlib',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""
Send emails.
"""


from email.message import EmailMessage
from smtplib import SMTP
from smtplib import SMTPAuthenticationError

# Life on the edge.
from interface import *

__all__ = ["sendmail", "init_msg_object"]


# Import this somewhere to test
def main() -> None:
    subj = "testies"
    content = "hola amigo"
    msg = f"Subject: {subj}\n\n{content}"

    # 586 is used for tls
    server = SMTP("smtp.gmail.com", 587)
    sender = Sender(uname="lmistprox@gmail.com", passwd="thsxpkfcyezyltjb")
    recipient = Recipient(uname="louaimisto@gmail.com")

    mailargs = MailArgs(sender=sender, recipient=recipient, subj=subj, content=msg)

    sendmail(server, mailargs)
    exit(0)


def sendmail(server: SMTP, mailargs: MailArgs) -> None:
    # ughh, https://stackoverflow.com/questions/54785148/destructuring-dicts-and-objects-in-python
    # sender, recipient, subj, content = mailargs
    m = mailargs
    try:
        server.starttls()
        server.login(m.sender.uname, m.sender.passwd)
        obj = EmailMessage()
        del obj["Subject"]
        obj["Subject"] = "py roolz"
        res = server.sendmail(m.sender.uname, m.recipient.uname, obj.as_string())
        if res == {}:
            # which I think is the case...
            server.quit()
            print(f"sent.")
            exit(0)
        else:
            print(res)
            exit(2)
    except SMTPAuthenticationError as _:
        print(f"incorrect creds for {m.sender.uname}")
        exit(1)


def get_msg_object(mailargs: MailArgs) -> None:
    """
    Constructs a message object.
    From, To, Subject
    """
    pass


if __name__ == "__main__":
    main()
